@@ -50,15 +50,18 @@ int runsort(int argc, char **argv);
 int runstrstr(int argc, char **argv);
 int runlist(int argc, char **argv);
 int runmap(int argc, char **argv);
+int runheap(int argc, char **argv);
 int runwicked(int argc, char **argv);
 int runmisc(int argc, char **argv);
 int printfflush(const char *format, ...);
 int strpcmp(const void *ap, const void *bp);
+int intpcmp(const void *ap, const void *bp);
 int myrand(void);
 int dosort(int rnum, int disp);
 int dostrstr(int rnum, int disp);
 int dolist(int rnum, int disp);
 int domap(int rnum, int bnum, int disp);
+int doheap(int rnum, int max, int disp);
 int dowicked(int rnum);
 int domisc(void);
 
@@ -78,6 +81,8 @@ int main(int argc, char **argv){
     rv = runlist(argc, argv);
   } else if(!strcmp(argv[1], "map")){
     rv = runmap(argc, argv);
+  } else if(!strcmp(argv[1], "heap")){
+    rv = runheap(argc, argv);
   } else if(!strcmp(argv[1], "wicked")){
     rv = runwicked(argc, argv);
   } else if(!strcmp(argv[1], "misc")){
@@ -98,6 +103,7 @@ void usage(void){
   fprintf(stderr, "  %s strstr [-d] rnum\n", progname);
   fprintf(stderr, "  %s list [-d] rnum\n", progname);
   fprintf(stderr, "  %s map [-d] rnum [bnum]\n", progname);
+  fprintf(stderr, "  %s heap [-d] rnum [top]\n", progname);
   fprintf(stderr, "  %s wicked rnum\n", progname);
   fprintf(stderr, "  %s misc\n", progname);
   fprintf(stderr, "\n");
@@ -222,6 +228,41 @@ int runmap(int argc, char **argv){
 }
 
 
+/* parse arguments of heap command */
+int runheap(int argc, char **argv){
+  int i, rnum, max, disp, rv;
+  char *rstr, *mstr;
+  rstr = NULL;
+  mstr = NULL;
+  rnum = 0;
+  max = -1;
+  disp = FALSE;
+  for(i = 2; i < argc; i++){
+    if(argv[i][0] == '-'){
+      if(!strcmp(argv[i], "-d")){
+        disp = TRUE;
+      } else {
+        usage();
+      }
+    } else if(!rstr){
+      rstr = argv[i];
+    } else if(!mstr){
+      mstr = argv[i];
+    } else {
+      usage();
+    }
+  }
+  if(!rstr) usage();
+  rnum = atoi(rstr);
+  if(rnum < 1) usage();
+  if(mstr) max = atoi(mstr);
+  if(max < 0) max = rnum;
+  rv = doheap(rnum, max, disp);
+  rv = 0;
+  return rv;
+}
+
+
 /* parse arguments of wicked command */
 int runwicked(int argc, char **argv){
   int i, rnum, rv;
@@ -265,12 +306,15 @@ int printfflush(const char *format, ...){
 }
 
 
-/* comparing function */
+/* comparing function for strings */
 int strpcmp(const void *ap, const void *bp){
-  char *a, *b;
-  a = *(char **)ap;
-  b = *(char **)bp;
-  return strcmp(a, b);
+  return strcmp(*(char **)ap, *(char **)bp);
+}
+
+
+/* comparing function for integers */
+int intpcmp(const void *ap, const void *bp){
+  return *(int *)ap - *(int *)bp;
 }
 
 
@@ -287,7 +331,7 @@ int dosort(int rnum, int disp){
   char **ivector1, **ivector2, **ivector3, **ivector4, **ivector5;
   char buf[RECBUFSIZ];
   int i, len, err;
-  if(!disp) printfflush("<Sorting Test>\n  rnum=%d  disp=%d\n\n", rnum, disp);
+  if(!disp) printfflush("<Sorting Test>\n  rnum=%d\n\n", rnum);
   ivector1 = cbmalloc(rnum * sizeof(ivector1[0]));
   ivector2 = cbmalloc(rnum * sizeof(ivector2[0]));
   ivector3 = cbmalloc(rnum * sizeof(ivector3[0]));
@@ -376,10 +420,10 @@ int dostrstr(int rnum, int disp){
 /* perform list command */
 int dolist(int rnum, int disp){
   CBLIST *list;
-  int i, err, len;
-  char buf[RECBUFSIZ];
-  const char *val;
-  if(!disp) printfflush("<List Writing Test>\n  rnum=%d  disp=%d\n\n", rnum, disp);
+  const char *vbuf;
+  char buf[RECBUFSIZ], *tmp;
+  int i, err, len, vsiz;
+  if(!disp) printfflush("<List Writing Test>\n  rnum=%d\n\n", rnum);
   list = cblistopen();
   err = FALSE;
   for(i = 1; i <= rnum; i++){
@@ -395,14 +439,19 @@ int dolist(int rnum, int disp){
   }
   if(disp){
     for(i = 0; i < cblistnum(list); i++){
-      if((val = cblistval(list, i, NULL)) != NULL){
-        printfflush("%s\n", val);
+      if((vbuf = cblistval(list, i, &vsiz)) != NULL){
+        printfflush("%s:%d\n", vbuf, vsiz);
       } else {
         fprintf(stderr, "%s: val error\n", progname);
         err = TRUE;
         break;
       }
     }
+    printfflush("\n");
+    while((tmp = cblistpop(list, &vsiz)) != NULL){
+      printfflush("%s:%d\n", tmp, vsiz);
+      free(tmp);
+    }
   }
   cblistclose(list);
   if(!disp && !err) printfflush("ok\n\n");
@@ -413,10 +462,10 @@ int dolist(int rnum, int disp){
 /* perform list command */
 int domap(int rnum, int bnum, int disp){
   CBMAP *map;
-  int i, err, len;
+  const char *kbuf, *vbuf;
   char buf[RECBUFSIZ];
-  const char *val;
-  if(!disp) printfflush("<Map Writing Test>\n  rnum=%d  bnum=%d  disp=%d\n\n", rnum, bnum, disp);
+  int i, err, len, ksiz, vsiz;
+  if(!disp) printfflush("<Map Writing Test>\n  rnum=%d  bnum=%d\n\n", rnum, bnum);
   map = bnum > 0 ? cbmapopenex(bnum) : cbmapopen();
   err = FALSE;
   for(i = 1; i <= rnum; i++){
@@ -437,12 +486,18 @@ int domap(int rnum, int bnum, int disp){
   if(disp){
     for(i = 1; i <= rnum; i++){
       len = sprintf(buf, "%08d", i);
-      if((val = cbmapget(map, buf, len, NULL)) != NULL){
-        printfflush("%s\n", val);
+      if((vbuf = cbmapget(map, buf, len, &vsiz)) != NULL){
+        printfflush("%s:%d\t%s:%d\n", buf, len, vbuf, vsiz);
       } else {
         fprintf(stderr, "%s: get error\n", progname);
       }
     }
+    printfflush("\n");
+    cbmapiterinit(map);
+    while((kbuf = cbmapiternext(map, &ksiz)) != NULL){
+      vbuf = cbmapiterval(kbuf, &vsiz);
+      printfflush("%s:%d\t%s:%d\n", kbuf, ksiz, vbuf, vsiz);
+    }
   }
   cbmapclose(map);
   if(!disp && !err) printfflush("ok\n\n");
@@ -450,6 +505,49 @@ int domap(int rnum, int bnum, int disp){
 }
 
 
+/* perform heap command */
+int doheap(int rnum, int max, int disp){
+  CBHEAP *heap;
+  int *orig, *ary;
+  int i, err, num, anum;
+  if(!disp) printfflush("<Heap Writing Test>\n  rnum=%d  max=%d\n\n", rnum, max);
+  orig = disp ? cbmalloc(rnum * sizeof(int) + 1) : NULL;
+  heap = cbheapopen(sizeof(int), max, intpcmp);
+  err = FALSE;
+  for(i = 1; i <= rnum; i++){
+    num = myrand() % rnum + 1;
+    if(orig) orig[i-1] = num;
+    cbheapinsert(heap, &num);
+    if(!disp && rnum > 250 && i % (rnum / 250) == 0){
+      putchar('.');
+      fflush(stdout);
+      if(i == rnum || i % (rnum / 10) == 0){
+        printfflush(" (%08d)\n", i);
+      }
+    }
+  }
+  if(disp){
+    for(i = 0; i < cbheapnum(heap); i++){
+      printf("%d\n", *(int *)cbheapval(heap, i));
+    }
+    printf("\n");
+    qsort(orig, rnum, sizeof(int), intpcmp);
+    ary = (int *)cbheaptomalloc(heap, &anum);
+    if(anum != rnum && anum != max) err = TRUE;
+    for(i = 0; i < anum; i++){
+      printf("%d\t%d\n", ary[i], orig[i]);
+      if(ary[i] != orig[i]) err = TRUE;
+    }
+    free(ary);
+  } else {
+    cbheapclose(heap);
+  }
+  free(orig);
+  if(!disp && !err) printfflush("ok\n\n");
+  return err ? 1 : 0;
+}
+
+
 /* perform wicked command */
 int dowicked(int rnum){
   CBLIST *list;
@@ -522,11 +620,6 @@ int dowicked(int rnum){
       cbmapmove(map, buf, len, myrand() % 2);
       putchar('M');
       break;
-    case 5:
-      tmp = cbmemdup(buf, len);
-      cbmapputvbuf(map, buf, len, tmp, len);
-      putchar('B');
-      break;
     default:
       cbmapput(map, buf, len, buf, len, TRUE);
       putchar('O');
