@@ -161,6 +161,18 @@ void Curia::remove(const char* name) throw(Curia_error){
 }
 
 
+char* Curia::snaffle(const char* name, const char* kbuf, int ksiz, int* sp) throw(Curia_error){
+  char* vbuf;
+  if(pthread_mutex_lock(&mutex) != 0) throw Curia_error();
+  if(!(vbuf = crsnaffle(name, kbuf, ksiz, sp))){
+    pthread_mutex_unlock(&mutex);
+    throw Curia_error(dpecode);
+  }
+  pthread_mutex_unlock(&mutex);
+  return vbuf;
+}
+
+
 Curia::Curia(const char* name, int omode, int bnum, int dnum) throw(Curia_error)
   : ADBM(){
   if(pthread_mutex_lock(&mutex) != 0) throw Curia_error();
