@@ -160,6 +160,18 @@ void Depot::remove(const char* name) throw(Depot_error){
 }
 
 
+char* Depot::snaffle(const char* name, const char* kbuf, int ksiz, int* sp) throw(Depot_error){
+  char* vbuf;
+  if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
+  if(!(vbuf = dpsnaffle(name, kbuf, ksiz, sp))){
+    pthread_mutex_unlock(&mutex);
+    throw Depot_error(dpecode);
+  }
+  pthread_mutex_unlock(&mutex);
+  return vbuf;
+}
+
+
 Depot::Depot(const char* name, int omode, int bnum) throw(Depot_error)
   : ADBM(){
   if(pthread_mutex_lock(&mutex) != 0) throw Depot_error();
