@@ -17,6 +17,7 @@
 #include "xvilla.h"
 #include <new>
 #include <cstdlib>
+#include <ctime>
 
 extern "C" {
 #include <depot.h>
@@ -141,12 +142,18 @@ const int Villa::OTRUNC = VL_OTRUNC;
 const int Villa::ONOLCK = VL_ONOLCK;
 const int Villa::OLCKNB = VL_OLCKNB;
 const int Villa::OZCOMP = VL_OZCOMP;
+const int Villa::OYCOMP = VL_OYCOMP;
+const int Villa::OXCOMP = VL_OXCOMP;
 const int Villa::DOVER = VL_DOVER;
 const int Villa::DKEEP = VL_DKEEP;
 const int Villa::DCAT = VL_DCAT;
 const int Villa::DDUP = VL_DDUP;
+const int Villa::DDUPR = VL_DDUPR;
 const int Villa::JFORWARD = VL_JFORWARD;
 const int Villa::JBACKWARD = VL_JBACKWARD;
+const int Villa::CPCURRENT = VL_CPCURRENT;
+const int Villa::CPBEFORE = VL_CPBEFORE;
+const int Villa::CPAFTER = VL_CPAFTER;
 
 
 const char* Villa::version() throw(){
@@ -356,6 +363,28 @@ char* Villa::curval(int *sp) throw(Villa_error){
 }
 
 
+void Villa::curput(const char* vbuf, int vsiz, int cpmode) throw(Villa_error){
+  if(!villa) throw Villa_error();
+  if(pthread_mutex_lock(&mutex) != 0) throw Villa_error();
+  if(!vlcurput(villa, vbuf, vsiz, cpmode)){
+    pthread_mutex_unlock(&mutex);
+    throw Villa_error(dpecode);
+  }
+  pthread_mutex_unlock(&mutex);
+}
+
+
+void Villa::curout() throw(Villa_error){
+  if(!villa) throw Villa_error();
+  if(pthread_mutex_lock(&mutex) != 0) throw Villa_error();
+  if(!vlcurout(villa)){
+    pthread_mutex_unlock(&mutex);
+    throw Villa_error(dpecode);
+  }
+  pthread_mutex_unlock(&mutex);
+}
+
+
 void Villa::settuning(int lrecmax, int nidxmax, int lcnum, int ncnum) throw(Villa_error){
   if(!villa) throw Villa_error();
   if(pthread_mutex_lock(&mutex) != 0) throw Villa_error();
@@ -481,8 +510,8 @@ int Villa::inode() throw(Villa_error){
 }
 
 
-int Villa::mtime() throw(Villa_error){
-  int rv;
+time_t Villa::mtime() throw(Villa_error){
+  time_t rv;
   if(!villa) throw Villa_error();
   if(pthread_mutex_lock(&mutex) != 0) throw Villa_error();
   rv = vlmtime(villa);
